;=========================================================================
.model small, c
.386
.stack
;=========================================================================

include support.asm

;=========================================================================

.data
.code

;=========================================================================

;----------------------------------------------------------------
; In an interrupt handler the only segment register with a known
; value is CS. This data is placed in the code segment so it can
; be accessed from the handler using a CS segment override.
;----------------------------------------------------------------

    prevIsr8  dd 0
    tickCount dd 0

;=========================================================================

.startup

    ;-----------------------------------------------------
    ; Hook the system timer tick interrupt (interrupt 8).
    ;-----------------------------------------------------

    ;-------------------------------------------------------
    ; The Get Interrupt Vector function returns the current
    ; vector in ES:BX.
    ;-------------------------------------------------------

    mov   ax, 3508h
    int   21h
    mov   WORD PTR prevIsr8, bx
    mov   WORD PTR prevIsr8[2], es

    ;-------------------------------------------------------
    ; The Set Interrupt Vector function expects the address
    ; of the new handler in DS:DX.
    ;-------------------------------------------------------

    push  ds
    push  cs
    pop   ds
    mov   dx, OFFSET Isr8_0
    mov   ax, 2508h
    int   21h
    pop   ds

    call  waitkey

    print dword$(cs:tickCount),NL

    ;-----------------------------------------
    ; Unhook the system timer tick interrupt.
    ;-----------------------------------------

    push  ds
    mov   dx, WORD PTR prevIsr8
    mov   ds, WORD PTR prevIsr8[2]
    mov   ax, 2508h
    int   21h
    pop   ds

    call  waitkey

.exit

;=========================================================================

;---------------------------------------------------------------------
; The following are new handlers for the system timer tick interrupt.
; The CS override is necessary because the only segment register with
; a know value is CS, the data being accessed is in the code segment,
; and the instructions that access the data would otherwise use DS.
;---------------------------------------------------------------------

;--------------------------------------------------------------
; This handler chains to the previous handler with a simulated
; interrupt instruction. The handler at the end of the chain
; will issue an EOI to the interrupt controller (necessary for
; a hardware interrupt) and finish up by executing an IRET
; that will return control to this handler.
;--------------------------------------------------------------

Isr8_0 proc

    pushf
    cli
    call  cs:prevIsr8         ; ML will encode this as a far call

    ;------------------------
    ; Update the tick count.
    ;------------------------

    inc   cs:tickCount

    ;----------------------------
    ; Return from the interrupt.
    ;----------------------------

    iret

Isr8_0 endp

;=========================================================================

;--------------------------------------------------------------
; This handler chains to the previous handler with a far jump.
; As for the previous handler, the handler at the end of the
; chain will issue an EOI to the interrupt controller.
;--------------------------------------------------------------

Isr8_1 proc

    ;------------------------
    ; Update the tick count.
    ;------------------------

    inc   cs:tickCount

    ;----------------------------
    ; Return from the interrupt.
    ;----------------------------

    jmp   cs:prevIsr8         ; ML will encode this as a far jump

Isr8_1 endp

;=========================================================================

;----------------------------------------------------------
; This handler does not chain to the previous handler, but
; instead issues an EOI to the interrupt controller and
; returns from the interrupt
;----------------------------------------------------------

Isr8_2 proc

    ;------------------------
    ; Update the tick count.
    ;------------------------

    inc   cs:tickCount

    ;----------------------------------------------------------
    ; For a hardware interrupt the last handler in the handler
    ; chain must issue an EOI to the interrupt controller.
    ;----------------------------------------------------------

    cli
    push ax
    mov al, 20h
    out 20h, al
    pop ax

    iret

Isr8_2 endp

;=========================================================================

end

